@@ -29,6 +29,42 @@
 #  include <linux/soundcard.h>
 #endif
 
+typedef struct mixer_channel_ident_t
+{
+  char * channel_name;
+  int    channel_id;
+} mixer_channel_ident;
+
+mixer_channel_ident channel_idents[] =
+  {
+    { "volume",    SOUND_MIXER_VOLUME	},
+    { "bass",      SOUND_MIXER_BASS	},
+    { "treble",    SOUND_MIXER_TREBLE	},
+    { "synth",     SOUND_MIXER_SYNTH	},
+    { "pcm",       SOUND_MIXER_PCM	},
+    { "speaker",   SOUND_MIXER_SPEAKER	},
+    { "line",      SOUND_MIXER_LINE	},
+    { "mic",       SOUND_MIXER_MIC	},
+    { "cd",        SOUND_MIXER_CD	},
+    { "imix",      SOUND_MIXER_IMIX	},
+    { "pcm2",      SOUND_MIXER_ALTPCM	},
+    { "record",    SOUND_MIXER_RECLEV	},
+    { "igain",     SOUND_MIXER_IGAIN	},
+    { "ogain",     SOUND_MIXER_OGAIN	},
+    { "line1",     SOUND_MIXER_LINE1	},
+    { "line2",     SOUND_MIXER_LINE2	},
+    { "line3",     SOUND_MIXER_LINE3	},
+    { "digital1",  SOUND_MIXER_DIGITAL1	},
+    { "digital2",  SOUND_MIXER_DIGITAL2	},
+    { "digital3",  SOUND_MIXER_DIGITAL3	},
+    { "phone-in",  SOUND_MIXER_PHONEIN	},
+    { "phone-out", SOUND_MIXER_PHONEOUT	},
+    { "video",     SOUND_MIXER_VIDEO	},
+    { "radio",     SOUND_MIXER_RADIO	},
+    { "monitor",   SOUND_MIXER_MONITOR	},
+    { NULL, 0 }
+  };
+
 typedef struct stereovolume
 {
   unsigned char left;
@@ -112,13 +148,14 @@ static char *help_message[] = {
 "    -shape                  without groundplate",
 "    -iconic                 start up as icon",
 "    -withdrawn              start up in withdrawn mode",
+"    -channel <channel-id>   specify the channel to control (def: volume)",
 "    -V                      version control",
 NULL
 };
 
 void version()
 {
-	printf("asmix: AfterStep sound mixer volume control 1.3\n");
+	printf("asmix: AfterStep sound mixer volume control 1.4\n");
 }
 
 void usage()
@@ -251,23 +288,11 @@ int main(int argc,char *argv[])
   Geometry = "";
 
 
-  Master.mixer_id = SOUND_MIXER_VOLUME;
-  
-  mixer_fd = open ("/dev/mixer", O_RDWR, 0);   
-  if (mixer_fd < 0) {
-    fprintf (stderr,"Error opening mixer device");
-    exit (1);
-  }
-        
-  Master.supported=SOUND_MIXER_VOLUME;
-
-  if (ioctl(mixer_fd, SOUND_MIXER_READ_DEVMASK, &Master.supported) == -1)
-    Master.supported = 0xffff; /* Assume all are supported */
-/*  printf("Master.supported = 0x%x\n",Master.supported);*/
+  Master.mixer_id  = SOUND_MIXER_VOLUME;
+  Master.supported = SOUND_MIXER_VOLUME;
 
   /* Parse command line options */
   ProgName = argv[0];
-  
 
   for(i=1;i<argc;i++) {
     char *arg= argv[i];
@@ -292,6 +317,25 @@ int main(int argc,char *argv[])
       case 'w':
 	WITHDRAWN=1;
 	continue;
+      case 'c':
+	if(++i >=argc) usage();
+	{
+	  int cur=0;
+	  while(1)
+	  {
+	    if ( channel_idents[cur].channel_name == NULL )
+	      usage();
+	    if ( strcmp( channel_idents[cur].channel_name, argv[i] ) != 0 )
+	    {
+	      cur++;
+	      continue;
+	    }
+	    Master.mixer_id  = channel_idents[cur].channel_id;
+	    Master.supported = channel_idents[cur].channel_id;
+	    break;
+	  }
+	}
+	continue;
       case 'V':
 	version();
 	exit(0);
@@ -302,6 +346,17 @@ int main(int argc,char *argv[])
     }
   }
 
+  mixer_fd = open ("/dev/mixer", O_RDWR, 0);   
+  if (mixer_fd < 0) {
+    fprintf (stderr,"Error opening mixer device");
+    exit (1);
+  }
+
+  if (ioctl(mixer_fd, SOUND_MIXER_READ_DEVMASK, &Master.supported) == -1)
+    Master.supported = 0xffff; /* Assume all are supported */
+/*  printf("Master.supported = 0x%x\n",Master.supported);*/
+
+
   /* Open the display */
   if (!(dpy = XOpenDisplay(display_name)))  
     { 
